package com.newriders.jxta.chapter11;

import java.util.Enumeration;
import java.io.IOException;

import java.util.Hashtable;

import java.awt.Container;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownServiceException;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;

import javax.swing.border.Border;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.jxta.document.Advertisement;

import net.jxta.exception.PeerGroupException;
import net.jxta.exception.ServiceNotFoundException;

import net.jxta.id.ID;
import net.jxta.id.IDFactory;

import net.jxta.impl.util.BidirectionalPipeService;

import net.jxta.peergroup.PeerGroup;

import net.jxta.platform.Application;
import net.jxta.platform.ModuleClassID;

import net.jxta.protocol.PipeAdvertisement;

import com.newriders.jxta.chapter11.chat.ChatListener;
import com.newriders.jxta.chapter11.chat.ChatService;

import com.newriders.jxta.chapter11.impl.protocol.InitiateChatResponse;

import com.newriders.jxta.chapter11.presence.PresenceListener;
import com.newriders.jxta.chapter11.presence.PresenceService;

import com.newriders.jxta.chapter11.protocol.InitiateChatRequestMessage;
import com.newriders.jxta.chapter11.protocol.InitiateChatResponseMessage;
import com.newriders.jxta.chapter11.protocol.PresenceAdvertisement;


/**
 * A user interface application to show a buddy list and their current 
 * status using the Presence service. The application also allows the user 
 * to initiate a chat session using the Chat service.
 */
public class BuddyList extends JFrame implements Application, 
    ActionListener, PresenceListener
{
    /**
     * The Module Class ID for the BuddyList application.
     */
    public static final String refModuleClassID = 
        "urn:jxta:uuid-E340D55F97E141C9B46FB2B108D8C2B705";
    
    /**
     * The Module Specification ID for the BuddyList application.
     */
    public static final String refModuleSpecID = 
        "urn:jxta:uuid-E340D55F97E141C9B46FB2B108D8C2B7"
        + "581B0312E66046B6BB96BF6A2EC5F27906";

    /**
     * A list to display the set of "approved" buddies and their 
     * presence status.
     */
    private JList buddies = new JList();

    /**
     * The data model for the list widget.
     */
    private DefaultListModel buddiesData = new DefaultListModel();
    
    /**
     * The peer group to which the service belongs.
     */
    private PeerGroup peerGroup = null;

    /**
     * The Presence service used to update other users of this user's
     * current presence status.
     */
    private PresenceService presence = null;

    /**
     * The Chat service used to handle requesting chat sessions and
     * respond with approvals.
     */
    private ChatService chat = null;

    /**
     * A set of buddy email addresses, indexed by display name.
     */
    private Hashtable buddyNames = new Hashtable();

    /**
     * A set of buddy display names, indexed by email address.
     */
    private Hashtable buddyEmailAddresses = new Hashtable();

    /**
     * A set of buddy Peer IDs, indexed by email address.
     */
    private Hashtable buddyPeerIDs = new Hashtable();

    /**
     * The local user's email address.
     */
    private String emailAddress = null;

    /**
     * The local user's display name.
     */
    private String displayName = null;

    /**
     * The local user's current presence status.
     */
    private int presenceStatus = PresenceService.OFFLINE;

    /**
     * A handler to use for handling chat requests and approvals.
     */
    private ChatHandler chatHandler = new ChatHandler();

     
    /**
     * A simple menu handler to deal with the "Set Name..." menu item.
     */
    public class SetNameHandler implements ActionListener
    {
        /**
         * Handles the "Set Name..." menu item.
         *
         * @param   e the event for the menu item.
         */
        public void actionPerformed(ActionEvent e)
        {
            String newDisplayName = JOptionPane.showInputDialog(null, 
                "Enter a new display name :", 
                "Set Display Name...", JOptionPane.QUESTION_MESSAGE);

            if ((null != newDisplayName) && (0 < newDisplayName.length()))
            {
                // Announce change in presence status.
                presence.announcePresence(
                    presenceStatus, emailAddress, newDisplayName);

                displayName = newDisplayName;
            }
        }
    }    
    
    /**
     * A simple menu handler to deal with the "Add Buddy..." menu item.
     */
    public class AddBuddyHandler implements ActionListener
    {
        /**
         * Handles the "Add Buddy..." menu item.
         *
         * @param   e the event for the menu item.
         */
        public void actionPerformed(ActionEvent e)
        {
            String buddy = JOptionPane.showInputDialog(null, 
                "Enter the email address of your buddy:", 
                "Add Buddy...", JOptionPane.QUESTION_MESSAGE);

            if ((null != buddy) && (0 < buddy.length()))
            {
                // Ensure that the buddy isn't already in our list.
                if (null == buddyEmailAddresses.get(buddy))
                {
                    // We should really validate the email address, but
                    // for simplicity we'll just add it to the list
                    // marking the buddy as "offline". Use the email
                    // address as the buddy display name until we discover
                    // presence information.
                    add(buddy, buddy, PresenceService.OFFLINE);
                    
                    // Find the presence information for the buddy.
                    presence.findPresence(buddy);
                }
                else
                {
                    JOptionPane.showMessageDialog(null, 
                        "A buddy with that email address already exists!",
                        "Buddy Exists!", JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    }
    
    /**
     * A handler to deal with the application dialog being closed or the 
     * "Quit" menu item being activated.
     */
    public class QuitHandler extends WindowAdapter implements ActionListener
    {
        /**
         * Handles the "Quit" menu item.
         *
         * @param   e the event for the menu item.
         */
        public void actionPerformed(ActionEvent e)
        {
            quit();
        }

        /**
         * Handles the window's system Close button.
         *
         * @param   e the event for the window closing.
         */    
        public void windowClosing(WindowEvent e)
        {
            quit();
        }

        /**
         * Quits the application.
         */
        private void quit()
        {
            // Announce change in presence status.
            presenceStatus = PresenceService.OFFLINE;
            presence.announcePresence(
                presenceStatus, emailAddress, displayName);
            
            System.exit(0);
        }
    }

    /**
     * Handles the local user updating presence information.
     */
    public class StatusChangeHandler implements ActionListener
    {
        /**
         * Handles the "My Status" submenu items.
         *
         * @param   e the event for the menu item.
         */
        public void actionPerformed(ActionEvent e)
        {
            int newPresenceStatus = PresenceService.OFFLINE;
            String presenceString = 
                ((JCheckBoxMenuItem) e.getSource()).getText();

            if (presenceString.equals("Offline"))
            {
                newPresenceStatus = PresenceService.OFFLINE;
            }
            else if (presenceString.equals("Online"))
            {
                newPresenceStatus = PresenceService.ONLINE;
            }
            else if (presenceString.equals("Busy"))
            {
                newPresenceStatus = PresenceService.BUSY;
            }
            else if (presenceString.equals("Away"))
            {
                newPresenceStatus = PresenceService.AWAY;
            }

            if (newPresenceStatus != presenceStatus)
            {
                // Announce change in presence status.
                presence.announcePresence(
                    newPresenceStatus, emailAddress, displayName);
                presenceStatus = newPresenceStatus;
            }
        }
    }

    /**
     * A simple handler to deal with spawning a chat window when a chat
     * request is approved or for handling incoming chat requests.
     */
    public class ChatHandler implements ChatListener
    {
        /**
         * Handles an approval for a previously generated chat request.
         * Displays the ChatDialog and handles establishing the two-
         * way communication channel.
         *
         * @param   response the response object containing the Pipe
         *          Advertisement to use to establish two-way communication.
         */
        public void chatApproved(InitiateChatResponseMessage response)
        {
            ChatDialog chatDialog = null;

            // Extract the Pipe Advertisement from the chat response.
            PipeAdvertisement pipeAdv = response.getPipeAdvertisement();
            
            if (null != pipeAdv)
            {
                // Create a bidirectional pipe.
                BidirectionalPipeService pipeService = 
                    new BidirectionalPipeService(peerGroup);
                BidirectionalPipeService.Pipe pipe = null;
                String buddyName = null;

                while (null == pipe)
                {
                    // We just loop here.
                    try
                    {
                        pipe = pipeService.connect(pipeAdv, 120000);
                    }
                    catch (IOException e)
                    {
                        // Do nothing.
                        System.out.println("Connect error:" + e);
                    }
                }

                // Get the buddy's display name.
                buddyName = response.getName();
                if (buddyName == null)
                {
                    buddyName = response.getEmailAddress();
                }
                 
                // Create the conversation GUI, and show it.
                chatDialog = new ChatDialog(buddyName, displayName,
                    peerGroup.getPipeService(), pipe.getInputPipe(),
                    pipe.getOutputPipe());
                chatDialog.show();
            }
            else
            {
                JOptionPane.showMessageDialog(null, 
                    "Buddy's reply is missing pipe advertisement!", 
                    "Unable To Chat!", JOptionPane.ERROR_MESSAGE);
            }
        }

        /**
         * Handles an incoming request for a chat session. Checks that
         * the incoming request comes from a known buddy and responds
         * with an approval message. Also prepares two-way communications
         * and the chat user interface.
         * 
         * @param   request the request for a chat session.
         * @param   queryID the query ID to be used to send a response
         *          using the Resolver service.
         */
        public void chatRequested(InitiateChatRequestMessage request, 
            int queryID)
        {
            String buddyEmailAddress = request.getEmailAddress();

            // Check who is making the request against the list of
            // approved chat buddies.
            if (null != buddyEmailAddresses.get(buddyEmailAddress))
            {
                ChatDialog chatDialog = null;
                String buddyName = null;

                // If the request is part of the approved buddies, then
                // approve the chat request.
                BidirectionalPipeService pipeService = 
                    new BidirectionalPipeService(peerGroup);
                BidirectionalPipeService.Pipe pipe = null;
                
                // Create an accept pipe to use to create an input pipe and 
                // listen for connections.
                try
                {
                    BidirectionalPipeService.AcceptPipe acceptPipe =
                        pipeService.bind("JXTA Messenger Pipe");

                    // Extract the Pipe Advertisement and the Pipe ID.
                    PipeAdvertisement pipeAdv =
                        acceptPipe.getAdvertisement();

                    // Send the approval response.
                    chat.approveChat(pipeAdv, emailAddress, 
                        displayName, queryID);
                    
                    // "Accept" a connection, meaning set up the input pipe 
                    // and listen for messages. Set this object as the 
                    // MessageListener so that we can handle incoming 
                    // messages without having to spawn a thread to call
                    // waitForMessage on the input pipe.
                    while (null == pipe)
                    {
                        // We just loop here.
                        try
                        {
                            pipe = acceptPipe.accept(1200000);
                        }
                        catch (InterruptedException e)
                        {
                            // Do nothing.
                            System.out.println("Interrupted: " + e);
                        }
                    }

                    // Get the buddy's display name.
                    buddyName = request.getName();
                    if (buddyName == null)
                    {
                        buddyName = request.getEmailAddress();
                    }

                    // Create the conversation GUI.
                    chatDialog = new ChatDialog(buddyName, displayName,
                        peerGroup.getPipeService(), pipe.getInputPipe(),
                        pipe.getOutputPipe());
                }
                catch (IOException e2)
                {
                       System.out.println("Error in chatRequested: " + e2);
                }
            }
        }
    }

    /**
     * Creates a new BuddyList object.
     */
    public BuddyList()
    {
        super("JXTA Messenger");
    }

    /**
     * Handles the user interface event used to trigger 
     * a chat session with a buddy from the list.
     *
     * @param   e the event being handled.
     */
    public void actionPerformed(ActionEvent e)
    {
        if (null != chat)
        {
            if (-1 != buddies.getSelectedIndex())
            {
                String buddyEmailAddress = null;
                String peerID = null;
                String buddyName = 
                    (String) buddiesData.get(buddies.getSelectedIndex());

                // Extract the actual buddy name (without the 
                // presence string).
                buddyName = buddyName.substring(0, buddyName.indexOf(" ("));
                
                // Figure out the email address and Peer ID.
                buddyEmailAddress = (String) buddyNames.get(buddyName);
                peerID = (String) buddyPeerIDs.get(buddyEmailAddress);

                if ((null != buddyEmailAddress) && (null != peerID))
                {
                    // It would be best to time out due to lack of approval
                    // for chat session, but for simplicity we'll just
                    // hope that the request gets through. In a full-fledged 
                    // application, it would be better to time out and retry.
                    chat.requestChat(peerID, emailAddress, displayName,
                       chatHandler);
                }
                else
                {
                    JOptionPane.showMessageDialog(null, 
                        "Could not find buddy!", "Unable To Find Buddy",
                        JOptionPane.ERROR_MESSAGE);
                }
            }
            else
            {
                JOptionPane.showMessageDialog(null, 
                    "No buddy selected!", "Select A Buddy First!", 
                    JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    /**
     * Add a buddy to the list of buddies.
     *
     * @param   buddyDisplayName the display name for the buddy.
     * @param   buddyEmailAddress the email address for the buddy.
     * @param   status the initial presence status for the buddy.
     */
    public void add(String buddyDisplayName, String buddyEmailAddress, 
        int status)
    {
        String buddyStatus;
        
        buddyNames.put(buddyDisplayName, buddyEmailAddress);
        buddyEmailAddresses.put(buddyEmailAddress, buddyDisplayName);

        // Construct the list entry from both the displayName and
        // the current status.
        buddyStatus = buddyDisplayName + " ";
        switch (status)
        {
            case PresenceService.OFFLINE:
            {
                buddyStatus += "(Offline)";
                break;
            }
            
            case PresenceService.ONLINE:
            {
                buddyStatus += "(Online)";
                break;
            }
            
            case PresenceService.BUSY:
            {
                buddyStatus += "(Busy)";
                break;
            }
            
            case PresenceService.AWAY:
            {
                buddyStatus += "(Away)";
                break;
            }

            default:
            {
                buddyStatus += "(Offline)";
                break;
            }
        }
        buddiesData.addElement(buddyStatus);
    }

    /**
     * Initialize the BuddyList application for the peer group.
     *
     * @param       group the peer group containing this application.
     * @param       ID the assigned ID for the application.
     * @param       implAdv the Module Implementation Advertisement for
     *             the BuddyList application.
     * @exception   PeerGroupException never thrown in this implementation.
     */
    public void init(PeerGroup group, ID assignedID, Advertisement implAdv) 
        throws PeerGroupException
    {
        this.peerGroup = group;

        // Initialize the user interface for the buddy list.
        initializeUserInterface();
    }

    /**
     * Initialize the menu for the BuddyList user interface.
     */
    public JMenuBar initializeMenu()
    {
        JMenuBar menuBar = new JMenuBar();
        JMenu actionsMenu = new JMenu("Actions");
        JMenu myStatusMenuItem = new JMenu("My Status");
        JCheckBoxMenuItem offline = new JCheckBoxMenuItem("Offline");
        JCheckBoxMenuItem online = new JCheckBoxMenuItem("Online", true);
        JCheckBoxMenuItem busy = new JCheckBoxMenuItem("Busy");
        JCheckBoxMenuItem away = new JCheckBoxMenuItem("Away");
        JMenuItem addBuddyMenuItem = new JMenuItem("Add Buddy...");
        JMenuItem chatBuddyMenuItem = new JMenuItem("Chat With Buddy...");
        JMenuItem setNameMenuItem = new JMenuItem("Set Name...");
        JMenuItem quitMenuItem = new JMenuItem("Quit");
        ButtonGroup group = new ButtonGroup();

        // Configure the status menu.
        myStatusMenuItem.add(offline);
        group.add(offline);
        myStatusMenuItem.add(online);
        group.add(online);
        myStatusMenuItem.add(busy);
        group.add(busy);
        myStatusMenuItem.add(away);
        group.add(away);
        
        // Configure the listeners for the menu items.
        addBuddyMenuItem.addActionListener(new AddBuddyHandler());
        chatBuddyMenuItem.addActionListener(this);
        setNameMenuItem.addActionListener(new SetNameHandler());
        quitMenuItem.addActionListener(new QuitHandler());
        offline.addActionListener(new StatusChangeHandler());
        online.addActionListener(new StatusChangeHandler());
        busy.addActionListener(new StatusChangeHandler());
        away.addActionListener(new StatusChangeHandler());
        
        // Add the menu items to the menus.
        actionsMenu.add(myStatusMenuItem);
        actionsMenu.add(addBuddyMenuItem);
        actionsMenu.add(chatBuddyMenuItem);
        actionsMenu.add(setNameMenuItem);
        actionsMenu.addSeparator();
        actionsMenu.add(quitMenuItem);

        // Add the menus to the menu bar.
        menuBar.add(actionsMenu);

        return menuBar;
    }

    /**
     * Initialize the main user interface for the BuddyList application.
     */
    public void initializeUserInterface()
    {
        Container framePanel = getContentPane();

        // Set a border for the list of buddies.
        Border border = BorderFactory.createTitledBorder("Buddies");
        
        // Configure menu.
        setJMenuBar(initializeMenu());
        
        // Add the initialized inner panel to the frame panel.
        framePanel.add(buddies);
        
        // Pack the frame, preparing it for display.
        pack();
        setSize(200, 300);

        // Add a listener to handle quitting the application when
        // the window is closed using the system menu.
        addWindowListener(new QuitHandler());
        
        // Add ourselves as a listener to the list, and set
        // the model to supply the data for the list.
        buddies.setModel(buddiesData);
    }

    /**
     * Handles updating the user interface when new presence
     * information arrives.
     *
     * @param   presenceInfo the Presence Advertisement containing the
     *          newly arrived presence information.
     */
    public void presenceUpdated(PresenceAdvertisement presenceInfo)
    {
        String buddyName = null;

        // First check that this buddy is someone we're interested
        // in displaying presence information for.
        buddyName = 
            (String) buddyEmailAddresses.get(
                presenceInfo.getEmailAddress());
        if (null != buddyName)
        {
            // Add the Peer ID to our Hashtable of Peer IDs.
            buddyPeerIDs.put(presenceInfo.getEmailAddress(),
                presenceInfo.getPeerID());

            // Update the list entry.
            for (int i = 0; i < buddiesData.getSize(); i++)
            {
                String currentBuddy = (String) buddiesData.get(i);

                // See if this is the right buddy to update.
                if (currentBuddy.indexOf(buddyName) != -1)
                {
                    // Update the buddy name (in case it changed).
                    String buddyDisplayName = presenceInfo.getName();
                    String buddyEmailAddress =
                        presenceInfo.getEmailAddress();
                    if (null == buddyDisplayName)
                    {
                        buddyDisplayName = buddyEmailAddress;
                    }

                    // Fix hashtables to reflect changes in name.
                    buddyNames.remove(buddyName);
                    buddyNames.put(buddyDisplayName, buddyEmailAddress);
                    buddyEmailAddresses.remove(buddyEmailAddress);
                    buddyEmailAddresses.put(buddyEmailAddress,
                        buddyDisplayName);

                    // Update the list element to reflect the 
                    // presence change.
                    buddyDisplayName += " ";
                    switch (presenceInfo.getPresenceStatus())
                    {
                        case PresenceService.OFFLINE:
                        {
                            buddyDisplayName += "(Offline)";
                            break;
                        }
                        
                        case PresenceService.ONLINE:
                        {
                            buddyDisplayName += "(Online)";
                            break;
                        }
                        
                        case PresenceService.BUSY:
                        {
                            buddyDisplayName += "(Busy)";
                            break;
                        }
                        
                        case PresenceService.AWAY:
                        {
                            buddyDisplayName += "(Away)";
                            break;
                        }

                        default:
                        {
                            buddyDisplayName += "(Offline)";
                            break;
                        }
                    }
                    buddiesData.set(i, buddyDisplayName);
                }
            }
        }
    }

    /**
     * Starts the BuddyList application.
     *
     * @param   args the arguments to use to start the application.
     * @return  a status value.
     */
    public int startApp(String[] args)
    {
        int result = 0;

        try
        {
            // Find the Presence service on the peer group.
            ModuleClassID classID = (ModuleClassID) IDFactory.fromURL(
                new URL((PresenceService.refModuleClassID)));
            presence = (PresenceService) peerGroup.lookupService(classID);

            // Register ourselves as a presence listener.
            presence.addListener(this);

            // Find the Chat service on the peer group.
            classID = (ModuleClassID) IDFactory.fromURL(
                new URL((ChatService.refModuleClassID)));
            chat = (ChatService) peerGroup.lookupService(classID);

            // Register ourselves as a chat request listener.
            chat.addListener(chatHandler);

            // Prompt the user to enter his own user information.
            do
            {
                emailAddress = JOptionPane.showInputDialog(null, 
                    "What is your email address?", 
                    "Configuration: Step 1 of 2", 
                    JOptionPane.QUESTION_MESSAGE);
            } while (null == emailAddress);

            do
            {
                displayName = JOptionPane.showInputDialog(null, 
                    "Enter a display name:", 
                    "Configuration: Step 2 of 2", 
                    JOptionPane.QUESTION_MESSAGE);
            } while (null == displayName);

            // Announce that we are online.
            presenceStatus = PresenceService.ONLINE;
            presence.announcePresence(presenceStatus, emailAddress,
                displayName);
            
            // Show the user interface.
            initializeUserInterface();
            setVisible(true);
        }
        catch (Exception e)
        {
            result = 1;
        }

        return result;
    }

    /**
     * Stop the application.
     */
    public void stopApp()
    {
        if (null != presence)
        {
            // Remove ourselves as a presence listener.
            presence.removeListener(this);

            presence = null;
        }

        if (null != chat)
        {
            // Remove ourselves as a chat request listener.
            chat.removeListener(chatHandler);

            chat = null;
        }
        
        // Hide the user interface.
        setVisible(false);
    }
}