package com.newriders.jxta.chapter9;

import java.io.IOException;

import net.jxta.endpoint.EndpointAddress;
import net.jxta.endpoint.EndpointMessenger;
import net.jxta.endpoint.EndpointService;
import net.jxta.endpoint.Message;

import net.jxta.exception.PeerGroupException;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupFactory;


/**
 * A simple Endpoint client that sends a message directly to a 
 * service named EndpointServer concatenated with the Peer 
 * Group ID, with service parameters 012345 on a specific peer
 * located at an Endpoint Address using an EndpointMessenger.
 */
public class EndpointMessengerClient
{
    /**
     * The peer group for the application.
     */
    private PeerGroup peerGroup = null;

    /**
     * The service name to use when listening for messages.
     * This service name will be appended with the Peer Group ID
     * of the peer group when the JXTA platform is started.
     */
    private String serviceName = "EndpointServer";

    /**
     * The service parameters to use when listening for
     * messages.
     */
    private String serviceParameters = "012345";

    
    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform
     *             can't be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        peerGroup = PeerGroupFactory.newNetPeerGroup();

        // Add the Peer Group ID of the group to the service
        // name so the message is sent to the version of the
        // endpoint listener specific to this peer group.
        serviceName += peerGroup.getPeerGroupID().toString();
    }

    /**
     * Runs the application: starts the JXTA platform, accepts
     * user input message and endpoint info, and sends the
     * message to the Endpoint Address specified.
     *
     * @param   args the command-line arguments passed to the
     *     application.
     */
    public static void main(String[] args)
    {
        EndpointMessengerClient client = 
            new EndpointMessengerClient();

        try
        {
            boolean done = false;
            String messageString = null;
            String addressString = null;

            // Initialize the JXTA platform.
            client.initializeJXTA();

            while (!done)
            {
                // Reset the strings.
                addressString = null;
                messageString = null;

                // Get the message; if the message is ., then
                // quit the application.
                System.out.print("Enter a message (or '.' " 
                    + " to quit): ");
                messageString = client.readInput();

                if ((messageString.length() > 0) 
                    && (!messageString.equals(".")))
                {
                    // Get the destination Endpoint Address 
                    // from the user.
                    System.out.print(
                        "Enter an endpoint address: ");
                    while ((addressString == null) 
                        || (addressString.length() == 0))
                    {
                        addressString = client.readInput();
                    }

                    // Send a message to the server.
                    client.sendMessage(
                        messageString, addressString);
                }
                else
                {
                    // We're done.
                    done = true;
                }
            }

            // Stop the JXTA platform. Currently, there isn't
            // any nice way to do this.
            System.exit(0);
        }
        catch (PeerGroupException e)
        {
            System.out.println("Error starting JXTA platform: " 
                + e);
            System.exit(1);
        }
    }

    /**
     * Read a line of input from the system console.
     * 
     * @return  the String read from the System.in InputStream.
     */
    public String readInput()
    {
        StringBuffer result = new StringBuffer();
        boolean done = false;
        int character;

        while (!done)
        {
            try
            {
                // Read a character.
                character = System.in.read();

                // Check to see if the character is a newline.
                if ((character == -1) 
                    || ((char) character == '\n'))
                {
                    done = true;
                }
                else
                {
                    // Add the character to the result string.
                    result.append((char) character);
                }
            }
            catch (IOException e )
            {
                done = true;
            }
        }

        return result.toString().trim();
    }

    /**
     * Sends a message. In this case, the message string is sent
     * to the Endpoint Address specified, provided that the
     * Endpoint Address responds to a ping.
     *
     * @param   messageString the message to send to the peer.
     * @param   addressString the Endpoint Address of the
     *     destination peers.
     */
    public void sendMessage(String messageString, 
        String addressString)
    {
        EndpointService endpoint = 
            peerGroup.getEndpointService();
        EndpointAddress endpointAddress = 
            endpoint.newEndpointAddress(addressString);

        // Manipulate the Endpoint Address to include the
        // appropriate destination service name and parameters.
        endpointAddress.setServiceName(serviceName);
        endpointAddress.setServiceParameter(serviceParameters);

        // Check that we can reach the Endpoint Address.
        if (endpoint.ping(endpointAddress))
        {
            // Create a new message.
            Message message = endpoint.newMessage();

            // Populate the message contents with the
            // messageString.
            message.setString("MessageText", messageString);

            try
            {
                EndpointMessenger messenger = 
                    endpoint.getMessenger(endpointAddress);

                if (messenger != null)
                {
                    // Send the message directly to the Endpoint
                    // Address specified.
                    messenger.sendMessage(message);
                }
                else
                {
                    System.out.println("Unable to create "
                        + "messenger for given address.");
                }
            }
            catch (IOException e)
            {
                System.out.println("Error creating messenger "
                    + "or sending message: " + e);
            }
        }
        else
        {
            System.out.println("Unable to reach specified "
                + "address!");
        }
    }
}