package com.newriders.jxta.chapter9;

import java.io.IOException;

import net.jxta.endpoint.EndpointAddress;
import net.jxta.endpoint.EndpointService;
import net.jxta.endpoint.Message;

import net.jxta.exception.PeerGroupException;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupFactory;


/**
 * A simple client that uses the Endpoint service to propagate
 * messages to a service named EndpointServer concatenated
 * with the Peer Group ID, with the service parameters 012345
 * on all peers in the local LAN segment.
 */
public class EndpointPropagateClient
{
    /**
     * The peer group for the application.
     */
    private PeerGroup peerGroup = null;
 
    /**
     * The service name to use when listening for messages.
     * This service name will be appended with the Peer Group ID
     * of the peer group when the JXTA platform is started.
     */
    private String serviceName = "EndpointServer";

    /**
     * The service parameters to use when listening for
     * messages.
     */
    private String serviceParameters = "012345";

    
    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform
     *              can't be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        peerGroup = PeerGroupFactory.newNetPeerGroup();

        // Add the Peer Group ID of the group to the service
        // name so the message is sent to the version of the
        // endpoint listener specific to this peer group.
        serviceName += peerGroup.getPeerGroupID().toString();
    }

    /**
     * Runs the application: starts the JXTA platform, accepts
     * user input messages, and propagates them to other peers.
     *
     * @param   args the command-line arguments passed to the
     *     application.
     */
    public static void main(String[] args)
    {
        EndpointPropagateClient client = 
            new EndpointPropagateClient();

        try
        {
            boolean done = false;
            String messageString = null;

            // Initialize the JXTA platform.
            client.initializeJXTA();

            while (!done)
            {
                // Reset the message string.
                messageString = null;

                // Get the message; if the message is '.', 
                // then quit the application.
                System.out.print("Enter a message (or '.' " 
                    + "to quit): ");
                messageString = client.readInput();

                if ((messageString.length() > 0) 
                    && (!messageString.equals(".")))
                {
                    // Send a message to the server.
                    client.sendMessage(messageString);
                }
                else
                {
                    // We're done.
                    done = true;
                }
            }
               
            // Stop the JXTA platform. Currently, there isn't
            // any nice way to do this.
            System.exit(0);
        }
        catch (PeerGroupException e)
        {
            System.out.println("Error starting JXTA platform: "
                + e);
            System.exit(1);
        }
    }

    /**
     * Read a line of input from the system console.
     * 
     * @return  the String read from the System.in InputStream.
     */
    public String readInput()
    {
        StringBuffer result = new StringBuffer();
        boolean done = false;
        int character;

        while (!done)
        {
            try
            {
                // Read a character.
                character = System.in.read();

                // Check to see if the character is a newline.
                if ((character == -1) 
                    || ((char) character == '\n'))
                {
                    done = true;
                }
                else
                {
                    // Add the character to the result string.
                    result.append((char) character);
                }
            }
            catch (IOException e )
            {
                done = true;
            }
        }

        return result.toString().trim();
    }

    /**
     * Sends a message. In this case, the message string is
     * propagated to all peers in the peer group on the local
     * LAN segment.
     *
     * @param   messageString the message to send to other
     *          peers.
     */
    public void sendMessage(String messageString)
    {
        EndpointService endpoint = 
            peerGroup.getEndpointService();

        // Create a new message.
        Message message = endpoint.newMessage();

        // Populate the message contents with the messageString.
        message.setString("MessageText", messageString);

        try
        {
            // Propagate the message within the peer group.
            endpoint.propagate(message, serviceName,
                serviceParameters);
        }
        catch (IOException e)
        {
            System.out.println("Error sending message: " + e);
        }
    }
}