package com.newriders.jxta.chapter9;

import java.util.Enumeration;

import net.jxta.endpoint.EndpointAddress;
import net.jxta.endpoint.EndpointProtocol;
import net.jxta.endpoint.EndpointService;
import net.jxta.endpoint.EndpointListener;
import net.jxta.endpoint.Message;

import net.jxta.exception.PeerGroupException;

import net.jxta.impl.endpoint.Address;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupFactory;


/**
 * A simple server that listens on an endpoint, looking for 
 * Messages destined for a service named EndpointServer
 * concatenated with the Peer Group ID, with service 
 * parameters 012345.
 */
public class EndpointServer implements EndpointListener
{
    /**
     * The peer group for the application.
     */
    private PeerGroup peerGroup = null;

    /**
     * The service name to use when listening for messages.
     * This service name will be appended with the Peer Group ID
     * of the peer group when the JXTA platform is started.
     */
    private String serviceName = "EndpointServer";

    /**
     * The service parameters to use when listening for 
     * messages.
     */
    private String serviceParameters = "012345";

    
    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform
     *             can't be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        peerGroup = PeerGroupFactory.newNetPeerGroup();

        // Add the Peer Group ID of the group to the service
        // name so the endpoint listener is specific to 
        // the peer group.
        serviceName += peerGroup.getPeerGroupID().toString();
    }

    /**
     * Runs the application: starts the JXTA platform, starts
     * listening on the Endpoint service for messages.
     *
     * @param   args the command-line arguments passed to 
     *     the application.
     */
    public static void main(String[] args)
    {
        EndpointServer server = new EndpointServer();

        try
        {
            // Initialize the JXTA platform.
            server.initializeJXTA();

            // Start the server.
            server.start();
        }
        catch (PeerGroupException e)
        {
            System.out.println("Error starting JXTA platform: " 
                + e);
            System.exit(1);
        }
    }

    /**
     * The EndpointListener implementation. Accepts an incoming
     * message for processing.
     *
     * @param   message the Message that has arrived for
     *          processing.
     * @param   source the EndpointAddress of the peer sending
     *          the message.
     * @param   destination the EndpointAddress of the
     *          destination peer for the message.
     */
    public void processIncomingMessage(Message message,
        EndpointAddress source, EndpointAddress destination)
    {
        System.out.println("Message received from " + source 
            + " for " + destination + ":");
        System.out.println(message.getString("MessageText"));
    }

    /**
     * Start the server listening on the Endpoint service.
     */
    public void start()
    {
        EndpointService endpoint = 
            peerGroup.getEndpointService();

        // Print out all of the endpoint protocol addresses.
        // These can be used by the EndpointClient to send a
        // message to the EndpointServer.
        EndpointProtocol aProtocol = null;
        Enumeration protocols = endpoint.getEndpointProtocols();
        while (protocols.hasMoreElements())
        {
            aProtocol = 
                (EndpointProtocol) protocols.nextElement();

            // Print out the address.
            System.out.println("Endpoint address: " 
                + aProtocol.getPublicAddress().toString());
        }

        // Add ourselves as a listener to the Endpoint service.
        endpoint.addListener(serviceName + serviceParameters,
            this);
    }
}