package net.jxta.impl.shell.bin.example5_1;

import java.io.InputStream;
import java.io.IOException;
import java.io.StringWriter;

import java.util.Enumeration;

import net.jxta.document.Document;
import net.jxta.document.Element;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredDocument;
import net.jxta.document.StructuredDocumentFactory;
import net.jxta.document.StructuredTextDocument;
import net.jxta.document.TextElement;


/**
 * An example query message, which will be wrapped by a Resolver Query
 * Message to send the query to other peers. The query essentially asks
 * a simple math question: "What is the value of (base) raised to (power)?"
 */
public class ExampleQueryMsg
{
    /**
     * The base for query.
     */
    private double base = 0.0;

    /**
     * The power for the query.
     */
    private double power = 0.0;


    /**
     * Creates a query object using the given base and power.
     *
     * @param   aBase the base for the query.
     * @param   aPower the power for the query.
     */
    public ExampleQueryMsg(double aBase, double aPower)
    {
        super();

        this.base = aBase;
        this.power = aPower;
    }

    /**
     * Creates a query object by parsing the given input stream.
     *
     * @param       stream the InputStream source of the query data.
     * @exception   IOException if an error occurs reading the stream.
     */
    public ExampleQueryMsg(InputStream stream) throws IOException
    {
           StructuredTextDocument document = (StructuredTextDocument)
               StructuredDocumentFactory.newStructuredDocument(
                   new MimeMediaType("text/xml"), stream);

        Enumeration elements = document.getChildren();

        while (elements.hasMoreElements())
        {
            TextElement element = (TextElement) elements.nextElement();
            
            if(element.getName().equals("base"))
            {
                base = Double.valueOf(element.getTextValue()).doubleValue();
                continue;
            }
            
            if(element.getName().equals("power"))
            {
                power = Double.valueOf(
                    element.getTextValue()).doubleValue();
                continue;
            }
        }
    }

    /**
     * Returns the base for the query.
     *
     * @return  the base value for the query.
     */
    public double getBase()
    {
        return base;
    }

    /**
     * Returns a Document representation of the query.
     *
     * @param   asMimeType the desired MIME type representation for the
     *          query.
     * @return  a Document form of the query in the specified MIME
     *          representation.
     */
    public Document getDocument(MimeMediaType asMimeType)
    {
        StructuredDocument document = (StructuredTextDocument)
            StructuredDocumentFactory.newStructuredDocument(
                asMimeType, "example:ExampleQuery");
        Element element;

        element = document.createElement(
            "base", Double.toString(getBase()));
        document.appendChild(element);
        
        element = document.createElement("power",
            Double.toString(getPower()));
        document.appendChild(element);
        
        return document;
    }

    /**
     * Returns the power for the query.
     *
     * @return  the power value for the query.
     */
    public double getPower()
    {
        return power;
    }

    /**
     * Returns an XML String representation of the query.
     *
     * @return  the XML String representing this query.
     */
    public String toString()
    {
        try
        {
            StringWriter out = new StringWriter();
            StructuredTextDocument doc = (StructuredTextDocument)
                getDocument(new MimeMediaType("text/xml"));
            doc.sendToWriter(out);

            return out.toString();
        }
        catch (Exception e)
        {
            return "";
        }
    }
}