package com.newriders.jxta.chapter10;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import java.util.Vector;

import net.jxta.document.Advertisement;

import net.jxta.exception.PeerGroupException;
import net.jxta.exception.NoResponseException;
import net.jxta.exception.DiscardQueryException;
import net.jxta.exception.ResendQueryException;

import net.jxta.id.ID;

import net.jxta.impl.protocol.ResolverQuery;
import net.jxta.impl.protocol.ResolverResponse;

import net.jxta.peergroup.PeerGroup;

import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.protocol.ResolverQueryMsg;
import net.jxta.protocol.ResolverResponseMsg;

import net.jxta.resolver.QueryHandler;
import net.jxta.resolver.ResolverService;

import net.jxta.service.Service;


/**
 * The implementation of the ExampleService interface. This service
 * builds on top of the Resolver service to provide the query 
 * functionality.
 */
public class ExampleServiceImpl implements ExampleService, QueryHandler
{
    /**
     * The Module Implementation advertisement for this service.
     */
    private Advertisement implAdvertisement = null;

    /**
     * The handler name used to register the Resolver handler.
     */
    private String handlerName = null;

    /**
     * The set of listener objects registered with the service.
     */
    private Vector registeredListeners = new Vector();

    /**
     * The peer group to which the service belongs.
     */
    private PeerGroup myPeerGroup = null;

    /**
     * The Resolver service used to send response messages.
     */
    private ResolverService resolver = null;

    /**
     * A unique query ID that can be used to track a query.
     */
    private int queryID = 0;

    
    /**
     * Add a listener object to the service. When new ExampleResponseMsg 
     * responses arrive, the service will notify each registered listener.
     * This method is synchronized to prevent multiple threads from 
     * altering the set of registered listeners simultaneously.
     *
     * @param   listener the listener object to register with the service.
     */
    public synchronized void addListener(ExampleServiceListener listener)
    {
        registeredListeners.addElement(listener);
    }

    /**
     * Send a query to the network to determine the value of the given 
     * base raised to the given power.
     *
     * @param   base the base for the exponentiation operation.
     * @param   power the exponent for the exponentiation operation.
     */
    public void findAnswer(double base, double power)
    {
        // Make sure the service has been started.
        if (resolver != null)
        {
            // Create the query object using the given base and power.
            ExampleQueryMsg equery = new ExampleQueryMsg(base, power);
            String localPeerId = myPeerGroup.getPeerID().toString();

            // Wrap the query in a Resolver Query Message.
            // The following is the old API way of creating a ResolverResponse,
            // eliminated in the new (build 65e) stable release.
            //ResolverQuery query = new ResolverQuery(handlerName,
            //    "JXTACRED", localPeerId, equery.toString(), queryID++);
            // Here is the new way:
            ResolverQuery query = new ResolverQuery(handlerName,
                null, localPeerId, equery.toString(), queryID++);

            // Send the query using the Resolver service.     
                resolver.sendQuery(null, query);
        }
    }

    /**
     * Returns the advertisement for this service. In this case, this is 
     * the ModuleImplAdvertisement passed in when the service was 
     * initialized.
     *
     * @return  the advertisement describing this service.
     */
    public Advertisement getImplAdvertisement()
    {
        return implAdvertisement;
    }

    /**
     * Returns an interface used to protect this service.
     *
     * @return  the wrapper object to use to manipulate this service.
     */
    public Service getInterface()
    {
        // We don't really need to provide an interface object to protect
        // this service, so this method simply returns the service itself.
        return this;
    }

    /**
     * Initialize the service.
     *
     * @param       group the PeerGroup containing this service.
     * @param       assignedID the identifier for this service.
     * @param       implAdv the advertisement specifying this service.
     * @exception    PeerGroupException is not thrown ever by this
     *              implementation.
     */
    public void init(PeerGroup group, ID assignedID, Advertisement implAdv) 
        throws PeerGroupException
    {
        // Save the module's implementation advertisement.
        implAdvertisement = (ModuleImplAdvertisement) implAdv;

        // Use the assigned ID as the Resolver handler name.
        handlerName = assignedID.toString();

        // Save a reference to the group of which that this service 
        // is a part.
        myPeerGroup = group;
    }

    /**
     * Processes the Resolver query message and returns a response. 
     *
     * @param       query the message to be processed.
     * @exception   IOException if the query can't be read.
     */
    public ResolverResponseMsg processQuery(ResolverQueryMsg query) 
        throws IOException, NoResponseException, DiscardQueryException, 
            ResendQueryException
    {
        ResolverResponse response;
        ExampleQueryMsg eq;
        double answer = 0.0;

        try
        {
            // Extract the query message.
            eq = new ExampleQueryMsg(
                new ByteArrayInputStream((query.getQuery()).getBytes()));
        }
        catch (Exception e)
        {
            throw new IOException();
        }

        // Perform the calculation.
        answer = Math.pow(eq.getBase(), eq.getPower());
        
        // Create the response message.
        ExampleResponseMsg er = new ExampleResponseMsg(
            eq.getBase(), eq.getPower(), answer);

        // Wrap the response message in a Resolver Response Message.
        // The following is the old API way of creating a ResolverResponse,
        // eliminated in the new (build 65e) stable release.
        //response = new ResolverResponse(handlerName, "JXTACRED",
        //    query.getQueryId(), er.toString());
        // Here is the new way:
        response = new ResolverResponse(handlerName, null,
            query.getQueryId(), er.toString());

        // Return the message so that the Resolver service can handle 
        // sending it.
        return response;
    }

    /**
     * Process a Resolver Response Message.
     *
     * @param   response a response message to be processed.
     */
    public void processResponse(ResolverResponseMsg response)
    {
        ExampleResponseMsg er;
        ExampleServiceEvent event;

        
        try
        {
            // Extract the message from the Resolver response.
            er = new ExampleResponseMsg(
                new ByteArrayInputStream(
                    (response.getResponse()).getBytes()));

            // Create an event to send to the listeners.
            event = new ExampleServiceEvent(this, er);
             
            // Notify each of the registered listeners.
            if (registeredListeners.size() > 0)
            {
                ExampleServiceListener listener = null;

                for (int i = 0; i < registeredListeners.size(); i++)
                {
                    listener = (ExampleServiceListener)
                        registeredListeners.elementAt(i);
                    listener.processAnswer(event);
                }
            }
        }
        catch (Exception e)
        {
            // This is not the right type of response message, or
            // the message is improperly formed. Ignore the exception;
            // do nothing with the message.
        }
    }

    /**
     * Remove a given listener object from the service. Once removed, 
     * a listener will no longer be notified when a new ExampleResponseMsg
     * response arrives.
     *
     * @param   listener the listener object to unregister.
     */
    public synchronized void removeListener(ExampleServiceListener listener)
    {
        registeredListeners.removeElement(listener);
    }

    /**
     * Start the service.
     *
     * @param   args the arguments to the service. Not used.
     * @return  0 to indicate the service started.
     */
    public int startApp(String[] args)
    {
        // Now that the service is being started, set the ResolverService
        // object to use to handle queries and responses.
        resolver = myPeerGroup.getResolverService();

        // Add ourselves as a listener using the unique constructed
        // handler name.
        resolver.registerHandler(handlerName, this);
     
        return 0;
    }

    /**
     * Stop the service.
     */
    public void stopApp()
    {
        // Unregister ourselves as a listener.
        if (resolver != null)
        {
            resolver.unregisterHandler(handlerName);
        }
    }
}