package com.newriders.jxta.chapter11;

import java.util.Hashtable;

import java.net.MalformedURLException;
import java.net.UnknownServiceException;
import java.net.URL;

import net.jxta.discovery.DiscoveryService;

import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredDocument;

import net.jxta.exception.PeerGroupException;

import net.jxta.id.IDFactory;

import net.jxta.impl.peergroup.StdPeerGroupParamAdv;

import net.jxta.impl.protocol.EndpointAdv;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.peergroup.PeerGroupFactory;

import net.jxta.platform.ModuleClassID;
import net.jxta.platform.ModuleSpecID;

import net.jxta.protocol.ModuleClassAdvertisement;
import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.protocol.ModuleSpecAdvertisement;
import net.jxta.protocol.PeerGroupAdvertisement;

import com.newriders.jxta.chapter11.chat.ChatService;

import com.newriders.jxta.chapter11.impl.chat.ChatServiceImpl;

import com.newriders.jxta.chapter11.impl.presence.PresenceServiceImpl;

import com.newriders.jxta.chapter11.impl.protocol.PresenceAdv;

import com.newriders.jxta.chapter11.presence.PresenceService;

import com.newriders.jxta.chapter11.protocol.PresenceAdvertisement;


/**
 * The main class responsible for creating the chat peer group and starting 
 * the Chat and Presence services and the BuddyList application.
 */
public class JxtaMessenger
{
    /**
     * The Peer Group ID for the chat group.
     */
    private static final String refPeerGroupID = 
        "urn:jxta:uuid-68B8A7A691684F9C9E05971D66D78ED602";

    /**
     * The Module Specification ID for the peer group's Module 
     * Implementation Advertisement.
     */
    private static final String refPeerGroupSpec = 
        "urn:jxta:uuid-DEADBEEFDEAFBABAFEEDBABE000000019E2"
        + "DDBBCE5FE4957A139B0ECE8DEB46D06";

    /**
     * The new group created by the application.
     */
    private PeerGroup newGroup = null;
    
    /**
     * The Net Peer Group for the application.
     */
    private PeerGroup netPeerGroup = null;

    /**
     * Create the main application class.
     */
    public JxtaMessenger()
    {
        super();
    }

    /**
     * Creates a Module Class Advertisement using the given parameters.
     *
     * @param       moduleClassID the Module Class ID for the advertisement.
     * @param       name the symbolic name of the advertisement.
     * @param       description the description of the advertisement.
     * @exception   UnknownServiceException if the moduleClassID string 
     *             is malformed.
     * @exception   MalformedURLException if the moduleClassID string 
     *             is malformed.
     */
    private ModuleClassAdvertisement createModuleClassAdv(
        String moduleClassID, String name, String description) 
            throws UnknownServiceException, MalformedURLException
    {
        // Create the class ID from the refModuleClassID string.
        ModuleClassID classID = (ModuleClassID) IDFactory.fromURL(
            new URL((moduleClassID)));

        // Create the Module Class Advertisement.
        ModuleClassAdvertisement moduleClassAdv = 
            (ModuleClassAdvertisement) 
                AdvertisementFactory.newAdvertisement(
                    ModuleClassAdvertisement.getAdvertisementType());

        // Configure the Module Class Advertisement.
        moduleClassAdv.setDescription(description);
        moduleClassAdv.setModuleClassID(classID);
        moduleClassAdv.setName(name);

        // Return the advertisement to the caller.
        return moduleClassAdv;
    }

    /**
     * Creates a Module Implementation Advertisement for the service using
     * the specification ID in the passed in ModuleSpecAdvertisement 
     * advertisement.  Use the given ModuleImplAdvertisement to create the 
     * compatibility element of the Module Implementation Advertisement.
     *
     * @param       groupImpl the ModuleImplAdvertisement of the parent 
     *             peer group.
     * @param       moduleSpecAdv the source of the specification ID.
     * @param       description of the module implementation.
     * @param       code the fully qualified name of the module
     *             implementation's class.
     * @return      the generated Module Implementation Advertisement.
     */
    private ModuleImplAdvertisement createModuleImplAdv(
        ModuleImplAdvertisement groupImpl, 
        ModuleSpecAdvertisement moduleSpecAdv, 
        String description, String code)
    {
        // Get the specification ID from the passed advertisement.
        ModuleSpecID specID = moduleSpecAdv.getModuleSpecID();

        // Create the Module Implementation Advertisement.
        ModuleImplAdvertisement moduleImplAdv = 
           (ModuleImplAdvertisement) AdvertisementFactory.newAdvertisement(
               ModuleImplAdvertisement.getAdvertisementType());

        // Configure the Module Implementation Advertisement.
        moduleImplAdv.setCode(code);
        moduleImplAdv.setCompat(groupImpl.getCompat());
        moduleImplAdv.setDescription(description);
        moduleImplAdv.setModuleSpecID(specID);
        moduleImplAdv.setProvider("Brendon J. Wilson");

        // Return the advertisement to the caller.
        return moduleImplAdv;
    }

    /**
     * Creates a Module Specification Advertisement using the 
     * given parameters.
     *
     * @param       moduleSpecID the Module Specification ID for 
     *             the advertisement.
     * @param       name the symbolic name of the advertisement.
     * @param       description the description of the advertisement.
     * @exception   UnknownServiceException if the moduleSpecID string 
     *             is malformed.
     * @exception   MalformedURLException if the moduleSpecID string 
     *             is malformed.
     */
    private ModuleSpecAdvertisement createModuleSpecAdv(String moduleSpecID,
        String name, String description) 
            throws UnknownServiceException, MalformedURLException
    {
        // Create the specification ID from the refModuleSpecID string.
        ModuleSpecID specID = (ModuleSpecID) IDFactory.fromURL(
            new URL((moduleSpecID)));

        // Create the Module Specification Advertisement.
        ModuleSpecAdvertisement moduleSpecAdv = 
            (ModuleSpecAdvertisement) AdvertisementFactory.newAdvertisement(
                ModuleSpecAdvertisement.getAdvertisementType());

        // Configure the Module Specification Advertisement.
        moduleSpecAdv.setCreator("Brendon J. Wilson");
        moduleSpecAdv.setModuleSpecID(specID);
        moduleSpecAdv.setDescription(description);
        moduleSpecAdv.setName(name);
        moduleSpecAdv.setSpecURI(
            "http://www.brendonwilson.com/projects/jxta");
        moduleSpecAdv.setVersion("1.0");

        // Return the advertisement to the caller.
        return moduleSpecAdv;
    }

    /**
     * Creates a peer group and configures the ChatService and
     * PresenceService implementations to run as peer group services,
     * and configures the BuddyList as a peer group application.
     *
     * @exception   Exception, PeerGroupException if there is a problem 
     *             while creating the peer group or the service
     *             advertisements.
     */
    public void createPeerGroup() throws Exception, PeerGroupException
    {
        // The name and description for the peer group.
        String name = "JXTA Messenger Group";
        String description = 
            "A peer group for the Chapter 11 example application.";

        // The Discovery service to use to publish the module and peer 
        // group advertisements.
        DiscoveryService discovery = netPeerGroup.getDiscoveryService();
        
        ModuleImplAdvertisement implAdv =
            netPeerGroup.getAllPurposePeerGroupImplAdvertisement();
        
        // Create the module advertisements for the Presence service.
        ModuleClassAdvertisement presenceClassAdv = createModuleClassAdv(
            PresenceService.refModuleClassID, "Presence Service", 
            "A service to provide presence information.");
        ModuleSpecAdvertisement presenceSpecAdv = createModuleSpecAdv(
            PresenceServiceImpl.refModuleSpecID, "Presence Service", 
            "A Presence service specification");
        ModuleImplAdvertisement presenceImplAdv = createModuleImplAdv(
            implAdv, presenceSpecAdv, 
            "The reference Presence service implementation",
            "com.newriders.jxta.chapter11.impl.presence."
            + "PresenceServiceImpl");

        // Create the module advertisements for the Chat service.
        ModuleClassAdvertisement chatClassAdv = createModuleClassAdv(
            ChatService.refModuleClassID, "Chat Service", 
            "A service to provide chat capabilities.");
        ModuleSpecAdvertisement chatSpecAdv = createModuleSpecAdv(
            ChatServiceImpl.refModuleSpecID, "Chat Service", 
            "A Chat service specification");
        ModuleImplAdvertisement chatImplAdv = createModuleImplAdv(
            implAdv, chatSpecAdv, 
            "The reference Chat service implementation",
            "com.newriders.jxta.chapter11.impl.chat.ChatServiceImpl");

        // Create the module advertisements for the BuddyList application.
        ModuleClassAdvertisement appClassAdv = createModuleClassAdv(
            BuddyList.refModuleClassID , "BuddyList Application", 
            "An application providing a simple chat application.");
        ModuleSpecAdvertisement appSpecAdv = createModuleSpecAdv(
            BuddyList.refModuleSpecID, "BuddyList Application", 
            "The BuddyList application specification");
        ModuleImplAdvertisement appImplAdv = createModuleImplAdv(
            implAdv, appSpecAdv ,
            "The reference BuddyList application implementation",
            "com.newriders.jxta.chapter11.BuddyList");
        
        // Get the parameters for the peer group's Module Implementation 
        // Advertisement to which we will add our service.
        StdPeerGroupParamAdv params = 
            new StdPeerGroupParamAdv(implAdv.getParam());

        // Get the services from the parameters.
        Hashtable services = params.getServices();

        // Add the Chat and Presence services to the set of services.
        services.put(presenceClassAdv.getModuleClassID(), presenceImplAdv);
        services.put(chatClassAdv.getModuleClassID(), chatImplAdv);

        // Set the services on the parameters.
        params.setServices(services);

        // Replace the applications in the parameters.
        Hashtable applications = new Hashtable();

        // Add the BuddyList to the applications.
        applications.put(appClassAdv.getModuleClassID(), appImplAdv);

        // Set the applications on the parameters.
        params.setApps(applications);
        
        // Set the parameters on the implementation advertisement.
        implAdv.setParam((StructuredDocument) params.getDocument(
            new MimeMediaType("text", "xml")));

        // VERY IMPORTANT! You must change the module specification ID for 
        // the implementation advertisement. If you don't, the new peer 
        // group's module specification ID will still point to the old 
        // specification, and the new service will not be loaded.
        implAdv.setModuleSpecID((ModuleSpecID) IDFactory.fromURL(
            new URL(refPeerGroupSpec)));
        
        // Publish the Presence module class and spec advertisements.
        discovery.publish(presenceClassAdv, DiscoveryService.ADV);
        discovery.remotePublish(presenceClassAdv, DiscoveryService.ADV);
        discovery.publish(presenceSpecAdv, DiscoveryService.ADV);
        discovery.remotePublish(presenceSpecAdv, DiscoveryService.ADV);
        
        // Publish the Presence module class and spec advertisements.
        discovery.publish(chatClassAdv, DiscoveryService.ADV);
        discovery.remotePublish(chatClassAdv, DiscoveryService.ADV);
        discovery.publish(chatSpecAdv, DiscoveryService.ADV);
        discovery.remotePublish(chatSpecAdv, DiscoveryService.ADV);

        // Publish the Peer Group implementation advertisement.
        discovery.publish(implAdv, DiscoveryService.ADV);
        discovery.remotePublish(implAdv, DiscoveryService.ADV);
        
        // Create the Peer Group ID.
        PeerGroupID groupID = (PeerGroupID) IDFactory.fromURL(
            new URL((refPeerGroupID)));;

        // Create the new group using the group ID, advertisement, name, 
        // and description.
        newGroup = netPeerGroup.newGroup(groupID, implAdv, name,
            description);

        // Need to publish the group remotely only because newGroup()
        // handles publishing to the local peer.
        PeerGroupAdvertisement groupAdv = 
            newGroup.getPeerGroupAdvertisement();
        discovery.remotePublish(groupAdv, DiscoveryService.GROUP);

        // Start the peer group's applications.
        newGroup.startApp(null);
    }

    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform can't 
     *             be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        netPeerGroup = PeerGroupFactory.newNetPeerGroup();
    }

    /**
     * Starts the application.
     * @param args an array of command-line arguments
     */
    public static void main(String[] args)
    {
        JxtaMessenger messenger = new JxtaMessenger();
        
        try
        {
            // Initialize the JXTA platform.
            messenger.initializeJXTA();

            // Create the group.
            messenger.createPeerGroup();
        }
        catch (Exception e)
        {
            System.out.println("Error starting JXTA platform: " + e);
            System.exit(1);
        }
    }
}