package net.jxta.impl.shell.bin.example5_1;

import net.jxta.impl.protocol.ResolverQuery;

import net.jxta.impl.shell.GetOpt;
import net.jxta.impl.shell.ShellApp;
import net.jxta.impl.shell.ShellEnv;
import net.jxta.impl.shell.ShellObject;

import net.jxta.peergroup.PeerGroup;

import net.jxta.resolver.QueryHandler;
import net.jxta.resolver.ResolverService;


/**
 * A simple application to demonstrate the use of the Resolver service
 * to register a QueryHandler instance and process queries.
 */
public class example5_1 extends ShellApp
{
    /**
     * A flag indicating if the example's handler should be unregistered
     * from the peer group's Resolver service.
     */     
    private boolean removeHandler = false;

    /**
     * A name to use to register the example handler with the 
     * Resolver service.
     */     
    private String name = "ExampleHandler";

    /**
     * The base value for the query.
     */
    private double base = 0.0;

    /**
     * The power value for the query.
     */
    private double power = 0.0;

     
    /**
     * Manages adding or removing the handler from the Resolver service.
     *
     * @param   resolver the Resolver service with which to register or
     *          unregister a handler.
     */
    private void manageHandler(ResolverService resolver)
    {
        if (removeHandler)
        {
            // Unregister the handler from the Resolver service.
            ExampleHandler handler = 
                (ExampleHandler) resolver.unregisterHandler(name);
        }
        else
        {
            // Create a new handler.
            ExampleHandler handler = new ExampleHandler();

            // Register the handler with the Resolver service.
            resolver.registerHandler(name, handler);
        }
    }

    /**
     * Parses the command-line arguments and initializes the command
     *
     * @param       args the arguments to be parsed.
     * @exception   IllegalArgumentException if an invalid parameter 
     *              is passed.
     */
    private void parseArguments(String[] args) 
        throws IllegalArgumentException
    {
        int option;

        // Parse the arguments to the command.
        GetOpt parser = new GetOpt(args, "b:p:r");

        while ((option = parser.getNextOption()) != -1)
        {
            switch (option)
            {
                case 'b' :
                {
                    try
                    {
                        // Obtain the "base" element for the query.
                        base = (new Double(
                            parser.getOptionArg())).doubleValue();
                    }
                    catch (Exception e)
                    {
                        // Default to 0.0
                        base = 0.0;
                    }

                    break;
                }
                
                case 'p' :
                {
                    try
                    {
                        // Obtain the "power" element for the query.
                        power = (new Double(
                            parser.getOptionArg())).doubleValue();
                    }
                    catch (Exception e)
                    {
                        // Default to 0.0
                        power = 0.0;
                    }

                    break;
                }
                
                case 'r' :
                {
                    // Remove the handler.
                    removeHandler = true;
                    break;
                }
            }
        }
    }

    /**
     * The implementation of the Shell command, invoked when the command 
     * is started by the user from the Shell.
     *
     * @param   args the command-line arguments passed to the command.
     * @return  a status code indicating the success or failure of 
     *          the command.
     */
    public int startApp(String[] args)
    {
        int result = appNoError;

        // Get the shell's environment.
        ShellEnv theEnvironment = getEnv();

        // Use the environment to obtain the current peer group.
        ShellObject theShellObject = theEnvironment.get("stdgroup");
        PeerGroup currentGroup = (PeerGroup) theShellObject.getObject();

        // Get the Resolver service for the current peer group.
        ResolverService resolver = currentGroup.getResolverService();

        try
        {
            // Parse the command-line arguments.
            parseArguments(args);
        }
        catch (IllegalArgumentException e)
        {
            println("Incorrect parameters passed to the command.");
            result = ShellApp.appParamError;
        }
        
        // Manage the handler for the Resolver service. This
        // adds or removes the handler as specified by the
        // command-line parameters.
        manageHandler(resolver);
        
        // If we're not removing the handler, send a query using
        // the Resolver service.
        if (!removeHandler)
        {
            ExampleQueryMsg equery = new ExampleQueryMsg(base, power);
            String localPeerId = currentGroup.getPeerID().toString();
            
            // Wrap the query in a resolver query.
            // The following is the old API way of creating a ResolverQuery,
            // eliminated in the new (build 65e) stable release.
            // ResolverQuery query = new ResolverQuery("ExampleHandler",
            //    "JXTACRED", localPeerId, equery.toString(), 0);
            // Here is the new way:
            ResolverQuery query = new ResolverQuery("ExampleHandler",
                null, localPeerId, equery.toString(), 0);

            // Send the query using the resolver.
            println("Sending base="+base+", power="+power);
            resolver.sendQuery(null, query);
        }
        
        return result;
    }
}