package net.jxta.impl.shell.bin.example7_2;

import java.io.IOException;
import java.io.StringWriter;

import java.net.URL;

import java.util.Enumeration;

import net.jxta.document.Advertisement;
import net.jxta.document.StructuredTextDocument;
import net.jxta.document.MimeMediaType;

import net.jxta.id.IDFactory;

import net.jxta.peer.PeerID;
import net.jxta.peer.PeerInfoListener;
import net.jxta.peer.PeerInfoService;

import net.jxta.peergroup.PeerGroup;

import net.jxta.protocol.PeerInfoResponseMessage;

import net.jxta.impl.shell.GetOpt;
import net.jxta.impl.shell.ShellApp;
import net.jxta.impl.shell.ShellEnv;
import net.jxta.impl.shell.ShellObject;


/**
 * A simple Shell command to demonstrate the use of the Peer Info
 * service and the PeerInfoListener interface to retrieve locally cached
 * status info.
 */
public class example7_2 extends ShellApp
{
    /**
     * The ID of the peer from which peer info is being solicited.
     */
    private String peerid = null;


    /**
     * Parses the command-line arguments and initializes the command
     *
     * @param       args the arguments to be parsed.
     * @exception   IllegalArgumentException if an invalid parameter 
     *              is passed.
     */
    private void parseArguments(String[] args) 
        throws IllegalArgumentException
    {
        int option;

        // Parse the arguments to the command.
        GetOpt parser = new GetOpt(args, "p:l");

        while ((option = parser.getNextOption()) != -1)
        {
            switch (option)
            {
                case 'p' :
                {
                     // Set the ID of the peer used to retrieve peer info.
                     peerid = parser.getOptionArg();

                    break;
                }
            }
        }
    }

    /**
     * Retrieves peer information from the local cache.
     *
     * @param   aPeerId the ID of the peer from which to solicit status info.
     * @param   peerinfo the PeerInfoService to use to perform the query,
     */
    private void sendLocalRequest(String aPeerId, PeerInfoService peerinfo)
    {
        try
        {
            PeerID peerIdObject = (PeerID) IDFactory.fromURL(
                new URL((aPeerId)));
            Enumeration enum = peerinfo.getLocalPeerInfo(peerIdObject);

            // Iterate through the response messages.
            while (enum.hasMoreElements())
            {
                // Extract the peer info response from the event.
                PeerInfoResponseMessage response = 
                    (PeerInfoResponseMessage) enum.nextElement();

                // Print out the peer info.
                System.out.println("Uptime: " + response.getUptime());
                System.out.println("Timestamp: " + response.getTimestamp());
                System.out.println("Target: " + response.getTargetPid());
                System.out.println("Source: " + response.getSourcePid());
                System.out.println("Last Incoming Message: " 
                    + response.getLastIncomingMessageTime());
                System.out.println("Last Outcoming Message: " 
                    + response.getLastOutgoingMessageTime());

                // Print out the incoming channel statistics.
                Enumeration incoming = 
                    response.getIncomingTrafficChannels();
                if (incoming != null)
                {
                    while (incoming.hasMoreElements())
                    {
                        String incomingchannel = 
                           (String) incoming.nextElement();
                        long incomingbytes =
                            response.getIncomingTrafficOnChannel(
                                incomingchannel);

                        System.out.println(incomingbytes 
                            + " incoming bytes on channel " 
                            + incomingchannel);
                    }
                }

                // Print out the outgoing channel statistics.
                Enumeration outgoing = 
                    response.getOutgoingTrafficChannels();
                if (outgoing != null)
                {
                    while (outgoing.hasMoreElements())
                    {
                        String outgoingchannel = 
                            (String) outgoing.nextElement();
                        long outgoingbytes =
                            response.getOutgoingTrafficOnChannel(
                                outgoingchannel);

                        System.out.println(outgoingbytes 
                            + " incoming bytes on channel " 
                            + outgoingchannel);
                    }
                }

                System.out.println("Done with status info...");
            }
        }
        catch (IOException e)
        {
            println("Error retrieving local peer info responses!" + e);
        }
    }

    /**
     * The implementation of the Shell command, invoked when the command 
     * is started by the user from the Shell.
     *
     * @param   args the command-line arguments passed to the command.
     * @return  a status code indicating the success or failure of 
     *          the command.
     */
    public int startApp(String[] args)
    {
        String peerid = null;
        int result = appNoError;

        // Get the shell's environment.
        ShellEnv theEnvironment = getEnv();

        // Use the environment to obtain the current peer group.
        ShellObject theShellObject = theEnvironment.get("stdgroup");
        PeerGroup currentGroup = (PeerGroup) theShellObject.getObject();

        // Get the PeerInfo service for the current peer group.
        PeerInfoService peerinfo = currentGroup.getPeerInfoService();

        // Default to getting the local peer's status info.
        peerid = currentGroup.getPeerID().toString();

        try
        {
            // Parse the command-line arguments.
            parseArguments(args);
        }
        catch (IllegalArgumentException e)
        {
            println("Incorrect parameters passed to the command.");
            result = ShellApp.appParamError;
        }

        peerid = null;

        // Send a local peer info request.
        System.out.println("Running example7_2...");
        sendLocalRequest(peerid, peerinfo);

        return result;
    }
}